/*
 Fuses:
    #BODLEVEL_1V8=10
    #F_CPU=128000
    avrdude -i 1000 -c usbasp -e -p t13 -U hfuse:w:0xfb:m -U lfuse:w:0x77:m

 Conections:
    ATtiny13a/1 = X(N.C.)
    ATtiny13a/2 = J2/4  [SEL]
    ATtiny13a/3 = J2/5  [HOLD]
    ATtiny13a/4 = GND(BAT-)
    ATtiny13a/5 = 24C08/5   [MUTE.TRIGGER]
    ATtiny13a/6 = J2/6  [SOC_RST]
    ATtiny13a/7 = Q4/BASE   [MUTE]
    ATtiny13a/8 = VCC       [J2/2]/[Q5/SOURCE]
    CAP_u1/1 = ATtiny13a/4
    CAP_u1/2 = ATtiny13a/8
*/


#include <avr/io.h>
#include <avr/interrupt.h>
//#include <avr/wdt.h>
#include <stdint.h>

#include <util/delay_basic.h>

#define F_CPU 128000UL
//#define BIT(q) (1 << q)

#define WAIT_1MS()      {_delay_loop_1(F_CPU/3/1000);}
#define WAIT_10MS()     {_delay_loop_2(F_CPU/4/100);}
#define WAIT_100MS()    {_delay_loop_2(F_CPU/4/10);}
#define WAIT_500MS()    {_delay_loop_2(F_CPU/4/2);}
#define WAIT_1S()       {_delay_loop_2(F_CPU/4);}
#define WAIT_2S()       {_delay_loop_2(F_CPU/2);}

#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80


// for disable APO
#define BTN_SEL_PIN     BIT4
// on/off key function
#define BTN_HOLD_PIN    BIT3
#define SOC_RST_PIN     BIT1
// disable buz on switch modes
#define MUTE_PIN        BIT2
#define I2C_PIN         BIT0

//#define DISABLE_APO_TIMEOUT 500
//#define BOOT_TIMEOUT        3000
#define MUTE_TIMEOUT    ((250)/10)


#define MUTE_ENABLE() { \
    PORTB &= ~MUTE_PIN; \
    DDRB |= MUTE_PIN; \
}
#define MUTE_DISABLE() { \
    PORTB &= ~MUTE_PIN; \
    DDRB &= ~MUTE_PIN; \
}

#define SEL_PULLUP_INPUT() { \
    DDRB &= ~BTN_SEL_PIN; \
    PORTB |= BTN_SEL_PIN; \
}
#define SEL_PUSH() { \
    PORTB &= ~BTN_SEL_PIN; \
    DDRB |= BTN_SEL_PIN; \
}
#define SEL_RELEASE() { \
    DDRB &= ~BTN_SEL_PIN; \
    PORTB &= ~BTN_SEL_PIN; \
}

#define HOLD_INIT() { \
    DDRB &= ~BTN_HOLD_PIN; \
    PORTB &= ~BTN_HOLD_PIN; \
}

#define DEVICE_ON() { \
    PORTB &= ~SOC_RST_PIN; \
    DDRB &= ~SOC_RST_PIN; \
}
#define DEVICE_OFF() { \
    PORTB &= ~SOC_RST_PIN; \
    DDRB |= SOC_RST_PIN; \
}

volatile uint8_t mute_cnt = 0;
// I2C isr(for mute switching)
ISR(PCINT0_vect) {
// disable buz on switch modes
    MUTE_ENABLE();
    GIMSK &= ~(1<<PCIE);// Disable next irq
    mute_cnt = MUTE_TIMEOUT;
}

/*
// HOLD/"PWR" button isr
ISR(INT0_vect) {
    GIMSK &= ~(1 << INT0);
TIM0_OVF Timer/Counter Overflow
TIM0_COMPA Timer/Counter Compare Match A
TIM0_COMPB
*/

void power_up() {
//    SEL_RELEASE();

// DISABLE APO: SEL_PIN=0 -> SOC_ON -> timeout -> SEL_PIN=release
//    SEL_PUSH();

// OFF CPU: RST release(if pullup present)
    DEVICE_ON();

//    WAIT_500MS();

// suppress second beep(SEL release?)
//    MUTE_ENABLE();

//    SEL_RELEASE();

// wait boot timeout
//    WAIT_1S();

//Release beep
//    MUTE_DISABLE();
}

void power_down() {
    __builtin_avr_cli();

//    HOLD_RELEASE();
//    SEL_PULLUP_INPUT();

    DEVICE_OFF();

    PCMSK = BTN_HOLD_PIN;
    GIMSK = (1 << PCIE);
    GIFR =  (1 << PCIE);

    MCUCR = (1 << SE) | (1 << SM1) | (0 << SM0); // Sleep ENABLE; PowerDown mode
    __builtin_avr_sei();
    __builtin_avr_sleep();
}

void power_down_wait_btn() {
// wait HOLD release
    while(!(PINB & BTN_HOLD_PIN)){}
}

//main Func
int main() {
    uint8_t pwr_tick_cnt = 0;
    HOLD_INIT();

    // Analog Comparator Disable
    ACSR = (1 << ACD);
    // Power Reduction Register
    PRR = (1 << PRTIM0) | (1 << PRADC);
    // DIDR0 – Digital Input Disable Register 0
    DIDR0 = (1 << MUTE_PIN);

    while(1) {
        power_up();

        PCMSK = I2C_PIN;
        GIFR |= (1<<PCIE);// Clear irq flag
        GIMSK |= (1<<PCIE);

        __builtin_avr_sei();
        pwr_tick_cnt = 0;
        while(1) {
            // MUTE routine
            if(GIFR & (1<<PCIE)) {
                MUTE_ENABLE();
                mute_cnt = MUTE_TIMEOUT;
                GIFR |= (1<<PCIE);
            }else{
                if(mute_cnt){
                    mute_cnt--;
                }else{
                    MUTE_DISABLE();
                    GIFR |= (1<<PCIE);// Clear irq flag
                    GIMSK |= (1<<PCIE);
                }
            }
            // PWR button routine
            if(!(PINB & BTN_HOLD_PIN)) {
                if(pwr_tick_cnt < 255) {
                    pwr_tick_cnt++;
                }
            }else{
                if(pwr_tick_cnt != 0) {
                    if((pwr_tick_cnt > 10) && (pwr_tick_cnt < 75)) { // Short press
                        break;
                    }
                    pwr_tick_cnt = 0;
                }
            }
            WAIT_10MS();
        }

        power_down();
        power_down_wait_btn();
    }

    return 0;
}

/*
// power down MCU
// PRR = 0xFF; // Power all down modules

    __builtin_avr_sei();
    __builtin_avr_cli();

    MCUCR = ( 1 << SE); // Sleep ENABLE; Idle mode
    __builtin_avr_sleep();
*/

